from fiscalsim_us.model_api import *


class ar_income_tax(Variable):
    "Line 38 of Form AR 1000F"
    value_type = float
    entity = TaxUnit
    label = "Arkansas income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.AR

    adds = ["ar_income_tax_before_refundable_credits"]
    subtracts = ["ar_refundable_credits"]



from fiscalsim_us.model_api import *


class pa_income_tax_before_forgiveness(Variable):
    value_type = float
    entity = TaxUnit
    label = "PA income tax before forgiveness"
    unit = USD
    definition_period = YEAR
    reference = "https://www.revenue.pa.gov/FormsandPublications/FormsforIndividuals/PIT/Documents/2021/2021_pa-40in.pdf#page=21"
    defined_for = StateCode.PA

    def formula(tax_unit, period, parameters):
        taxable_income = tax_unit("pa_adjusted_taxable_income", period)
        rate = parameters(period).gov.states.pa.tax.income.rate
        return taxable_income * rate



from fiscalsim_us.model_api import *


class sc_income_tax(Variable):
    """
    South Carolina income tax liability, from line 30 or line 34 of form SC1040
    2023 Individual Income Tax Return
    """

    value_type = float
    entity = TaxUnit
    label = "South Carolina income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.SC

    adds = ["sc_income_tax_before_refundable_credits"]
    subtracts = ["sc_refundable_credits"]



from fiscalsim_us.model_api import *


class nm_2021_income_rebate(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Mexico 2021 income tax rebate"
    definition_period = YEAR
    reference = "https://nmonesource.com/nmos/nmsa/en/item/4340/index.do#!fragment/zoupio-_Toc140503708/BQCwhgziBcwMYgK4DsDWszIQewE4BUBTADwBdoAvbRABwEtsBaAfX2zgEYAWABgFYeAZgDsPABwBKADTJspQhACKiQrgCe0AOSapEQmFwJlqjdt37DIAMp5SAIQ0AlAKIAZZwDUAggDkAws5SpGAARtCk7BISQA"
    defined_for = StateCode.NM

    def formula(tax_unit, period, parameters):
        dependent_on_another_return = tax_unit("dsi", period)
        income = tax_unit("adjusted_gross_income", period)
        p = (
            parameters(period)
            .gov.states.nm.tax.income.rebates["2021_income"]
            .main
        )
        filing_status = tax_unit("filing_status", period)
        income_eligible = income < p.income_limit[filing_status]
        eligible = income_eligible & ~dependent_on_another_return
        amount_if_eligible = p.amount[filing_status]
        return eligible * amount_if_eligible



from fiscalsim_us.model_api import *


class mt_gambling_loss_deduction(Variable):
    """
    Line 17 on itemized deductions schedule
    Limited to what it allowed under federal law
    """

    value_type = float
    entity = TaxUnit
    label = "Montana gambling loss deduction"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MT



from fiscalsim_us.model_api import *


class la_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana refundable credits"
    defined_for = StateCode.LA
    unit = USD
    definition_period = YEAR

    adds = "gov.states.la.tax.income.credits.refundable"



from fiscalsim_us.model_api import *


class ar_political_contribution_credit(Variable):
    "Line 1 of form AR1000TC"
    value_type = float
    entity = TaxUnit
    label = "Arkansas political contribution tax credit"
    unit = USD
    definition_period = YEAR
    reference = "https://www.dfa.arkansas.gov/images/uploads/incomeTaxOffice/2022_AR1000TC_Schedule_of_TaxCredits_andBusiness_IncentiveCredits.pdf"
    defined_for = StateCode.AR


def formula(tax_unit, period, parameters):
    p = parameters(period).gov.states.ar.tax.income.credits.other_credits
    cap = p.political_contrib
    contributions = tax_unit("ar_political_contributions")

    return min_(cap, contributions)



from fiscalsim_us.model_api import *


class co_tanf_countable_gross_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Colorado TANF countable gross unearned income"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.CO

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.co.cdhs.tanf.income
        # Sum unearned sources, plus child support if not currently enrolled.
        gross_unearned = add(spm_unit, period, p.unearned)
        child_support = add(spm_unit, period, ["child_support_received"])
        enrolled = spm_unit("is_tanf_enrolled", period)
        return gross_unearned + where(enrolled, 0, child_support)



from fiscalsim_us.model_api import *


class ia_amt_indiv(Variable):
    value_type = float
    entity = Person
    label = "Iowa alternative minimum tax when married couples file separately"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://tax.iowa.gov/sites/default/files/2023-01/2021%20Expanded%20Instructions_010323.pdf#page=55"
        "https://tax.iowa.gov/sites/default/files/2021-12/IA6251%2841131%29.pdf"
        "https://tax.iowa.gov/sites/default/files/2023-03/2022%20Expanded%20Instructions_022023.pdf#page=55"
        "https://tax.iowa.gov/sites/default/files/2023-01/IA6251%2841131%29.pdf"
    )
    defined_for = StateCode.IA

    def formula(person, period, parameters):
        # compute Iowa AMT taxable income
        reg_taxinc = person("ia_taxable_income_indiv", period)
        std_ded = person("ia_standard_deduction_indiv", period)
        itm_ded = person("ia_itemized_deductions_indiv", period)
        prorate_frac = person("ia_prorate_fraction", period)
        proptax = add(person.tax_unit, period, ["real_estate_taxes"])
        amt_taxinc = where(
            itm_ded > std_ded,
            reg_taxinc + prorate_frac * proptax,
            reg_taxinc,
        )
        # compute AMT amount
        p = parameters(period).gov.states.ia.tax.income
        amt = p.alternative_minimum_tax
        us_filing_status = person.tax_unit("filing_status", period)
        fsvals = us_filing_status.possible_values
        filing_status = select(
            [
                us_filing_status == fsvals.JOINT,
                us_filing_status == fsvals.SINGLE,
                us_filing_status == fsvals.SEPARATE,
                us_filing_status == fsvals.HEAD_OF_HOUSEHOLD,
                us_filing_status == fsvals.SURVIVING_SPOUSE,
            ],
            [
                fsvals.SEPARATE,  # couples are filing separately on Iowa form
                fsvals.SINGLE,
                fsvals.SEPARATE,
                fsvals.HEAD_OF_HOUSEHOLD,
                fsvals.SURVIVING_SPOUSE,
            ],
        )
        amt_threshold = amt.threshold[filing_status]  # Line 23
        amt_exemption = amt.exemption[filing_status]  # Line 24
        netinc = max_(0, amt_taxinc - amt_exemption)  # Line 25
        amount = max_(0, amt_threshold - netinc * amt.fraction)  # Line 27
        gross_amt = max_(0, amt_taxinc - amount) * amt.rate  # Line 29
        base_tax = person("ia_base_tax_indiv", period)  # Line 30
        return max_(0, gross_amt - base_tax)  # Line 31
